@@ -149,6 +149,7 @@ def get_ollama_response(
     Generates a response using the Ollama API, supporting both streaming and non-streaming.
     """
 
+    options = {}
 
     image_paths = []
     if images:
@@ -248,7 +249,6 @@ def get_ollama_response(
     elif isinstance(format, str) and format == "json" and not stream:
         api_params["format"] = "json"
 
-    options = {}
     for key, value in kwargs.items():
         if key in [
             "stop", 
@@ -552,6 +552,17 @@ def get_litellm_response(
         provider = "openai"
     
     
+    if provider =='enpisi' and api_url is None:
+        api_params['api_base'] = 'https://api.enpisi.com'
+        if api_key is None:
+            api_key = os.environ.get('NPC_STUDIO_LICENSE')
+            api_params['api_key'] = api_key
+            print(api_key)
+        if '-npc' in model: 
+            model = model.split('-npc')[0]
+        provider = "openai"
+
+    print(model, provider)
     if isinstance(format, BaseModel):
         api_params["response_format"] = format
     if model is None:
@@ -772,7 +783,7 @@ def process_tool_calls(response_dict, tool_map, model, provider, messages, strea
             
             result["messages"].append({
                 "role": "assistant",
-                "content": f'The results of the tool call for {tool_namee} with {arguments} are as follows:' +tool_result_str
+                "content": f'The results of the tool call for {tool_name} with {arguments} are as follows:' +tool_result_str
             })
     
     return result
\ No newline at end of file
