@@ -15,6 +15,8 @@ from npcpy.gen.response import get_litellm_response
 from npcpy.gen.image_gen import generate_image
 from npcpy.gen.video_gen import generate_video_diffusers, generate_video_veo3
 
+from datetime import datetime 
+
 def gen_image(
     prompt: str,
     model: str = None,
@@ -24,6 +26,8 @@ def gen_image(
     width: int = 1024,
     n_images: int=1, 
     input_images: List[Union[str, bytes, PIL.Image.Image]] = None,
+    save = False, 
+    filename = '',
 ):
     """This function generates an image using the specified provider and model.
     Args:
@@ -48,7 +52,7 @@ def gen_image(
         if npc.api_url is not None:
             api_url = npc.api_url
 
-    image = generate_image(
+    images = generate_image(
         prompt=prompt,
         model=model,
         provider=provider,
@@ -58,7 +62,14 @@ def gen_image(
         n_images=n_images, 
         
     )
-    return image
+    if save:
+        if len(filename) == 0 :
+            todays_date = datetime.now().strftime("%Y-%m-%d")
+            filename = 'vixynt_gen'
+        for i, image in enumerate(images):
+            
+            image.save(filename+'_'+str(i)+'.png')
+    return images
 
 
 def gen_video(
@@ -208,6 +219,8 @@ def get_llm_response(
     elif prompt:
         messages.append({"role": "user", 
                          "content": prompt + context_str})    
+    #import pdb 
+    #pdb.set_trace()
     response = get_litellm_response(
         prompt + context_str,
         messages=messages,
@@ -1663,6 +1676,9 @@ def get_facts(content_text,
               provider = None,
               npc=None,
               context : str=None, 
+              attempt_number=1,
+              n_attempts=3,
+
               **kwargs):
     """Extract facts from content text"""
     
@@ -1765,6 +1781,17 @@ def get_facts(content_text,
                                 context=context,
                                 **kwargs)
 
+    if len(response.get("response", {}).get("facts", [])) == 0 and attempt_number < n_attempts:
+        print(f"  Attempt {attempt_number} to extract facts yielded no results. Retrying...")
+        return get_facts(content_text, 
+                         model=model, 
+                         provider=provider, 
+                         npc=npc,
+                         context=context,
+                         attempt_number=attempt_number+1,
+                         n_attempts=n_attempts,
+                         **kwargs)
+    
     return response["response"].get("facts", [])
 
         
@@ -1774,6 +1801,8 @@ def zoom_in(facts,
             provider=None, 
             npc=None,
             context: str = None, 
+            attempt_number: int = 1,
+            n_attempts=3,            
             **kwargs):
     """Infer new implied facts from existing facts"""
     valid_facts = []
@@ -1794,16 +1823,16 @@ def zoom_in(facts,
     {facts_text}
 
     What other facts can be reasonably inferred from these?
-
+    """ +"""
     Respond with JSON:
-    {{
+    {
         "implied_facts": [
-            {{
+            {
                 "statement": "new implied fact",
                 "inferred_from": ["which facts this comes from"]
-            }}
+            }
         ]
-    }}
+    }
     """
     
     response = get_llm_response(prompt, 
@@ -1813,7 +1842,18 @@ def zoom_in(facts,
                                 context=context,
                                 npc=npc,
                                 **kwargs)
-    return response["response"].get("implied_facts", [])
+
+    facts =  response.get("response", {}).get("implied_facts", [])
+    if len(facts) == 0:
+        return zoom_in(valid_facts, 
+                       model=model, 
+                       provider=provider, 
+                       npc=npc,
+                       context=context,
+                       attempt_number=attempt_number+1,
+                       n_tries=n_tries,
+                       **kwargs)
+    return facts
 def generate_groups(facts, 
                     model=None,
                     provider=None,
@@ -1982,7 +2022,10 @@ def get_related_facts_llm(new_fact_statement,
                           model = None, 
                           provider = None,
                           npc = None, 
-                          context=''):
+                          attempt_number = 1,
+                          n_attempts = 3,
+                          context='', 
+                          **kwargs):
     """Identifies which existing facts are causally or thematically related to a new fact."""
     prompt = f"""
     A new fact has been learned: "{new_fact_statement}"
@@ -1995,12 +2038,25 @@ def get_related_facts_llm(new_fact_statement,
 
     Respond with JSON: {{"related_facts": ["statement of a related fact", ...]}}
     """
-    response = get_llm_response(prompt, 
+    response = get_llm_response(prompt,
                                 model=model, 
                                 provider=provider, 
-                                npc = npc,
                                 format="json", 
-                                context=context)
+                                npc=npc,
+                                context=context,
+                                **kwargs)   
+    if attempt_number > n_attempts:
+        print(f"  Attempt {attempt_number} to find related facts yielded no results. Giving up.")
+        return get_related_facts_llm(new_fact_statement, 
+                                       existing_fact_statements, 
+                                       model=model, 
+                                       provider=provider, 
+                                       npc=npc,
+                                       attempt_number=attempt_number+1,
+                                       n_attempts=n_attempts,
+                                       context=context,
+                                       **kwargs)    
+
     return response["response"].get("related_facts", [])
 
 def find_best_link_concept_llm(candidate_concept_name, 
