@@ -22,7 +22,8 @@ from npcpy.llm_funcs import (
     remove_idempotent_groups,
     zoom_in, 
     )
-from npcpy.npc_compiler import NPC
+
+from npcpy.memory.command_history import load_kg_from_db, save_kg_to_db
 
 def safe_kuzu_execute(conn, query, error_message="Kuzu query failed"):
     """Execute a Kuzu query with proper error handling"""
@@ -189,13 +190,14 @@ def find_similar_groups(
     return response["group_list"]
 
 
-def kg_initial(content_text=None,  
+def kg_initial(content,  
                model=None,
                provider=None,
                npc=None, 
                context='', 
                facts=None, 
-               generation=None):
+               generation=None, 
+               verbose=True,):
 
     if generation is None:
         CURRENT_GENERATION = 0
@@ -205,34 +207,99 @@ def kg_initial(content_text=None,
     print(f"--- Running KG Structuring Process (Generation: {CURRENT_GENERATION}) ---")
 
     if facts is None:
-        if not content_text:
+        if not content:
             raise ValueError("kg_initial requires either content_text or a list of facts.")
         print("  - Mode: Deriving new facts from text content...")
-        facts = get_facts(content_text, model=model, provider=provider, npc=npc, context=context)
-        for fact in facts:
+        all_facts = []
+        print(len(content))
+        if len(content)>10000:
+            # randomly sub sample 10000 characters
+            starting_point = random.randint(0, len(content)-10000)
+        
+            content_to_sample = content[starting_point:starting_point+10000]
+
+            for n in range(len(content)//10000):
+                print(n)
+                print(starting_point)
+                print(content_to_sample[0:1000])
+                facts = get_facts(content_to_sample,
+                                model=model, 
+                                provider=provider, 
+                                npc=npc, 
+                                context=context)
+                if verbose:
+                    print(f"    - Extracted {len(facts)} facts from segment {n+1}")
+                    print(facts)
+                all_facts.extend(facts)
+        else:
+            print(content[0:1000]   )
+            all_facts = get_facts(content, 
+                                  model=model, 
+                                  provider=provider, 
+                                  npc=npc, 
+                                  context=context)
+            if verbose:
+                print(f"    - Extracted {len(all_facts)} facts from content")
+                print(all_facts)            
+        for fact in all_facts:
+            
             fact['generation'] = CURRENT_GENERATION
     else:
         print(f"  - Mode: Building structure from {len(facts)} pre-existing facts...")
 
     print("  - Inferring implied facts (zooming in)...")
-    implied_facts = zoom_in(facts, model=model, provider=provider, npc=npc, context=context)
-    for fact in implied_facts:
+    all_implied_facts = []
+    if len(all_facts) > 20:
+        # sub sample facts randomly to generate zoomed in facts
+        sampled_facts = random.sample(all_facts, k=20)
+        for n in range(len(all_facts) // 20):
+            implied_facts = zoom_in(sampled_facts, 
+                                    model=model, 
+                                    provider=provider,
+                                npc=npc, 
+                                context=context)
+            all_implied_facts.extend(implied_facts)
+            if verbose:
+                print(f"    - Inferred {len(implied_facts)} implied facts from sample {n+1}")
+                print(implied_facts)
+    else:
+        implied_facts = zoom_in(all_facts, 
+                                model=model, 
+                                provider=provider,
+                                npc=npc, 
+                                context=context)
+        print(implied_facts)
+
+        all_implied_facts.extend(implied_facts)
+
+        if verbose:
+            print(f"    - Inferred {len(implied_facts)} implied facts from all facts")
+            print(implied_facts)
+    for fact in all_implied_facts:
         fact['generation'] = CURRENT_GENERATION
-    
-    all_facts = facts + implied_facts
-    
+
+    all_facts = all_facts + all_implied_facts
+
     print("  - Generating concepts from all facts...")
-    concepts = generate_groups(all_facts, model=model, provider=provider, npc=npc, context=context)
+    concepts = generate_groups(all_facts, 
+                               model=model, 
+                               provider=provider, 
+                               npc=npc, 
+                               context=context)
     for concept in concepts:
         concept['generation'] = CURRENT_GENERATION
-        
+    
+    if verbose:
+        print(f"    - Generated {len(concepts)} concepts")
+        print(concepts)
     print("  - Linking facts to concepts...")
     fact_to_concept_links = defaultdict(list)
     concept_names = [c['name'] for c in concepts if c and 'name' in c]
     for fact in all_facts:
 
         fact_to_concept_links[fact['statement']] = get_related_concepts_multi(fact['statement'], "fact", concept_names, model, provider, npc, context)
-        print(fact_to_concept_links[fact['statement']])
+        if verbose:
+            print(fact_to_concept_links[fact['statement']])
     print("  - Linking facts to other facts...")
     fact_to_fact_links = []
     fact_statements = [f['statement'] for f in all_facts]
@@ -249,7 +316,8 @@ def kg_initial(content_text=None,
             for related_stmt in related_fact_stmts:
 
                 fact_to_fact_links.append((fact['statement'], related_stmt))
-                print(fact['statement'], related_stmt)
+                if verbose:
+                    print(fact['statement'], related_stmt)
 
     return {
         "generation": CURRENT_GENERATION, 
@@ -283,7 +351,8 @@ def kg_evolve_incremental(existing_kg,
 
     newly_added_concepts = []
     concept_links = list(existing_kg.get('concept_links', []))
-    fact_to_concept_links = defaultdict(list, existing_kg.get('fact_to_concept_links', {}))
+    fact_to_concept_links = defaultdict(list, 
+                                        existing_kg.get('fact_to_concept_links', {}))
     fact_to_fact_links = list(existing_kg.get('fact_to_fact_links', []))
 
     existing_facts = existing_kg.get('facts', [])
@@ -291,23 +360,35 @@ def kg_evolve_incremental(existing_kg,
     existing_concept_names = {c['name'] for c in existing_concepts}
     existing_fact_statements = [f['statement'] for f in existing_facts]
     all_concept_names = list(existing_concept_names)
+    all_new_facts = []
+    if len(new_content_text) > 10000:
+        starting_point = random.randint(0, len(new_content_text)-10000)
+        for n in range(len(new_content_text)//10000):
+            content_to_sample = new_content_text[n*10000:(n+1)*10000]
+            facts = get_facts(content_to_sample, 
+                            model=model,
+                            provider=provider,
+                            npc = npc, 
+                            context=context)
+            all_new_facts.extend(facts)
+    else:
+        
+        all_new_facts = get_facts(new_content_text, 
+                            model=model,
+                            provider=provider,
+                            npc = npc, 
+                            context=context)
 
 
-    new_facts = get_facts(new_content_text, 
-                          model=model,
-                          provider=provider,
-                          npc = npc, 
-                          context=context)
-
-    for fact in new_facts: 
+    for fact in all_new_facts: 
         fact['generation'] = next_gen
-    
-    final_facts = existing_facts + new_facts
-    
+
+    final_facts = existing_facts + all_new_facts
+
     if get_concepts:
         print('generating groups...')
 
-        candidate_concepts = generate_groups(new_facts, 
+        candidate_concepts = generate_groups(all_new_facts, 
                                             model = model, 
                                             provider = provider, 
                                             npc=npc, 
@@ -339,21 +420,26 @@ def kg_evolve_incremental(existing_kg,
 
         if link_concepts_facts:
             print('linking facts and concepts...')
-            for fact in new_facts:
+            for fact in all_new_facts:
                 fact_to_concept_links[fact['statement']] = get_related_concepts_multi(fact['statement'], 
                                                                                     "fact", 
                                                                                     all_concept_names, 
-                                                                                    model, 
-                                                                                    provider,
-                                                                                    npc,  
-                                                                                    context)
+                                                                                    model = model,
+                                                                                    provider=provider,
+                                                                                    npc = npc, 
+                                                                                    context= context)
     else:
         final_concepts = existing_concepts
     if link_facts_facts:
         print('linking facts and facts...')
 
-        for new_fact in new_facts:
-            related_fact_stmts = get_related_facts_llm(new_fact['statement'], existing_fact_statements, model, provider, context)
+        for new_fact in all_new_facts:
+            related_fact_stmts = get_related_facts_llm(new_fact['statement'], 
+                                                       existing_fact_statements, 
+                                                       model = model,
+                                                       provider = provider,
+                                                       npc = npc, 
+                                                       context=context)
             for related_stmt in related_fact_stmts:
                 fact_to_fact_links.append((new_fact['statement'], related_stmt))
                 
@@ -371,7 +457,12 @@ def kg_evolve_incremental(existing_kg,
 
 
 
-def kg_sleep_process(existing_kg, model=None, provider=None, npc=None, context='', operations_config=None):
+def kg_sleep_process(existing_kg, 
+                     model=None, 
+                     provider=None, 
+                     npc=None, 
+                     context='', 
+                     operations_config=None):
     current_gen = existing_kg.get('generation', 0)
     next_gen = current_gen + 1
     print(f"\n--- SLEEPING (Evolving Knowledge): Gen {current_gen} -> Gen {next_gen} ---")
@@ -1049,3 +1140,271 @@ def save_facts_to_graph_db(
                 continue
 
         print(f"Completed batch {i//batch_size + 1}")
+
+
+
+def kg_add_fact(
+   engine, 
+   fact_text: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Add a new fact to the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   new_fact = {
+       "statement": fact_text,
+       "source_text": fact_text,
+       "type": "manual",
+       "generation": kg_data.get('generation', 0),
+       "origin": "manual_add"
+   }
+   
+   kg_data['facts'].append(new_fact)
+   save_kg_to_db(engine, kg_data, team_name, npc_name, directory_path)
+   
+   return f"Added fact: {fact_text}"
+
+def kg_search_facts(
+   engine,
+   query: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Search facts in the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   matching_facts = []
+   for fact in kg_data.get('facts', []):
+       if query.lower() in fact['statement'].lower():
+           matching_facts.append(fact['statement'])
+   
+   return matching_facts
+
+def kg_remove_fact(
+   engine,
+   fact_text: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Remove a fact from the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   original_count = len(kg_data.get('facts', []))
+   kg_data['facts'] = [f for f in kg_data.get('facts', []) if f['statement'] != fact_text]
+   removed_count = original_count - len(kg_data['facts'])
+   
+   if removed_count > 0:
+       save_kg_to_db(engine, kg_data, team_name, npc_name, directory_path)
+       return f"Removed {removed_count} matching fact(s)"
+   
+   return "No matching facts found"
+
+def kg_list_concepts(
+   engine,
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """List all concepts in the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   concepts = [c['name'] for c in kg_data.get('concepts', [])]
+   return concepts
+
+def kg_get_facts_for_concept(
+   engine,
+   concept_name: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Get all facts linked to a specific concept"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   fact_to_concept_links = kg_data.get('fact_to_concept_links', {})
+   linked_facts = []
+   
+   for fact_statement, linked_concepts in fact_to_concept_links.items():
+       if concept_name in linked_concepts:
+           linked_facts.append(fact_statement)
+   
+   return linked_facts
+
+def kg_add_concept(
+   engine,
+   concept_name: str, 
+   concept_description: str,
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Add a new concept to the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   new_concept = {
+       "name": concept_name,
+       "description": concept_description,
+       "generation": kg_data.get('generation', 0)
+   }
+   
+   kg_data['concepts'].append(new_concept)
+   save_kg_to_db(engine, kg_data, team_name, npc_name, directory_path)
+   
+   return f"Added concept: {concept_name}"
+
+def kg_remove_concept(
+   engine,
+   concept_name: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Remove a concept from the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   original_count = len(kg_data.get('concepts', []))
+   kg_data['concepts'] = [c for c in kg_data.get('concepts', []) if c['name'] != concept_name]
+   removed_count = original_count - len(kg_data['concepts'])
+   
+   if removed_count > 0:
+       save_kg_to_db(engine, kg_data, team_name, npc_name, directory_path)
+       return f"Removed concept: {concept_name}"
+   
+   return "Concept not found"
+
+def kg_link_fact_to_concept(
+   engine,
+   fact_text: str,
+   concept_name: str, 
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Link a fact to a concept in the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   fact_to_concept_links = kg_data.get('fact_to_concept_links', {})
+   
+   if fact_text not in fact_to_concept_links:
+       fact_to_concept_links[fact_text] = []
+   
+   if concept_name not in fact_to_concept_links[fact_text]:
+       fact_to_concept_links[fact_text].append(concept_name)
+       kg_data['fact_to_concept_links'] = fact_to_concept_links
+       save_kg_to_db(engine, kg_data, team_name, npc_name, directory_path)
+       return f"Linked fact '{fact_text}' to concept '{concept_name}'"
+   
+   return "Fact already linked to concept"
+
+def kg_get_all_facts(
+   engine,
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Get all facts from the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   facts = [f['statement'] for f in kg_data.get('facts', [])]
+   return facts
+
+def kg_get_stats(
+   engine,
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Get statistics about the knowledge graph"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   return {
+       "total_facts": len(kg_data.get('facts', [])),
+       "total_concepts": len(kg_data.get('concepts', [])),
+       "total_fact_concept_links": len(kg_data.get('fact_to_concept_links', {})),
+       "generation": kg_data.get('generation', 0)
+   }
+
+def kg_evolve_knowledge(
+   engine,
+   content_text: str,
+   npc=None, 
+   team=None,
+   model=None,
+   provider=None
+):
+   """Evolve the knowledge graph with new content"""
+   directory_path = os.getcwd()
+   team_name = getattr(team, 'name', 'default_team') if team else 'default_team'
+   npc_name = npc.name if npc else 'default_npc'
+   
+   kg_data = load_kg_from_db(engine, team_name, npc_name, directory_path)
+   
+   evolved_kg, _ = kg_evolve_incremental(
+       existing_kg=kg_data,
+       new_content_text=content_text,
+       model=npc.model if npc else model,
+       provider=npc.provider if npc else provider,
+       npc=npc,
+       get_concepts=True,
+       link_concepts_facts=False,
+       link_concepts_concepts=False,
+       link_facts_facts=False
+   )
+   
+   save_kg_to_db(engine, evolved_kg, team_name, npc_name, directory_path)
+   
+   return "Knowledge graph evolved with new content"
\ No newline at end of file
